@@ -27,7 +27,8 @@ class webull:
             'Accept-Encoding': 'gzip, deflate',
             'Content-Type': 'application/json',
             'platform': 'web',
-            'ver': '3.22.20',
+            'app': 'global',
+            'ver': '3.36.12',
             'User-Agent': '*',
             'lzone': 'dc_core_r001',
             'did': self._get_did(),
@@ -87,7 +88,7 @@ class webull:
         return headers
 
 
-    def login(self, username='', password='', device_name='', mfa='', question_id='', question_answer=''):
+    def login(self, username='', password='', device_name='', mfa='', question_id='', question_answer='', save_token=False, token_path=None):
         '''
         Login with email or phone number
 
@@ -138,10 +139,12 @@ class webull:
             self._token_expire = result['tokenExpireTime']
             self._uuid = result['uuid']
             self._account_id = self.get_account_id()
+            if save_token:
+                self._save_token(result, token_path)
         return result
 
     def get_mfa(self, username='') :
-        account_type = self.get_account_type()
+        account_type = self.get_account_type(username)
 
         data = {'account': str(username),
                 'accountType': str(account_type),
@@ -235,7 +238,7 @@ class webull:
         self._uuid = uuid
         self._account_id = self.get_account_id()
 
-    def refresh_login(self):
+    def refresh_login(self, save_token=False, token_path=None):
         '''
         Refresh login token
         '''
@@ -248,8 +251,23 @@ class webull:
             self._access_token = result['accessToken']
             self._refresh_token = result['refreshToken']
             self._token_expire = result['tokenExpireTime']
+            if save_token:
+                result['uuid'] = self._uuid
+                self._save_token(result, token_path)
         return result
 
+    def _save_token(self, token=None, path=None):
+        '''
+        save login token to webull_credentials.json
+        '''
+        filename = 'webull_credentials.json'
+        if path:
+            filename = os.path.join(path, filename)
+        with open(filename, 'wb') as f:
+            pickle.dump(token, f)
+            return True
+        return False
+
     def get_detail(self):
         '''
         get some contact details of your account name, email/phone, region, avatar...etc
@@ -261,7 +279,7 @@ class webull:
 
         return result
 
-    def get_account_id(self):
+    def get_account_id(self, id=0):
         '''
         get account id
         call account id before trade actions
@@ -270,11 +288,10 @@ class webull:
 
         response = requests.get(self._urls.account_id(), headers=headers)
         result = response.json()
-        if result['success'] :
-            self.zone_var = str(result['data'][0]['rzone'])
-            id = str(result['data'][0]['secAccountId'])
-            self._account_id = id
-            return id
+        if result['success'] and len(result['data']) > 0 :
+            self.zone_var = str(result['data'][int(id)]['rzone'])
+            self._account_id = str(result['data'][int(id)]['secAccountId'])
+            return self._account_id
         else:
             return None
 
@@ -446,7 +463,7 @@ class webull:
         response = requests.post(self._urls.place_orders(self._account_id), json=data, headers=headers)
         return response.json()
 
-    def modify_order(self, order=None, price=0, action=None, orderType=None, enforce=None, quant=0, outsideRegularTradingHour=None):
+    def modify_order(self, order=None, order_id=0, stock=None, tId=None, price=0, action=None, orderType=None, enforce=None, quant=0, outsideRegularTradingHour=None):
         '''
         Modify an order
 
@@ -455,8 +472,8 @@ class webull:
         timeinforce:  GTC / DAY / IOC
         outsideRegularTradingHour: True / False
         '''
-        if not order:
-            raise ValueError('Must provide an order')
+        if not order and not order_id:
+            raise ValueError('Must provide an order or order_id')
 
         headers = self.build_req_headers(include_trade_token=True, include_time=True)
 
@@ -466,6 +483,13 @@ class webull:
         modifiedOutsideRegularTradingHour = outsideRegularTradingHour if type(outsideRegularTradingHour) == bool else order['outsideRegularTradingHour']
         modifiedEnforce = enforce or order['timeInForce']
         modifiedQuant = int(quant or order['quantity'])
+        if not tId is None:
+            pass
+        elif not stock is None:
+            tId = self.get_ticker(stock)
+        else :
+            tId = order['ticker']['tickerId']
+        order_id = order_id or order['orderId']
 
         data = {
             'action': modifiedAction,
@@ -475,14 +499,15 @@ class webull:
             'comboType': 'NORMAL',
             'outsideRegularTradingHour': modifiedOutsideRegularTradingHour,
             'serialId': str(uuid.uuid4()),
-            'tickerId': order['ticker']['tickerId'],
+            'orderId': order_id,
+            'tickerId': tId,
             'timeInForce': modifiedEnforce
         }
         #Market orders do not support extended hours trading.
         if data['orderType'] == 'MKT':
             data['outsideRegularTradingHour'] = False
 
-        response = requests.put(self._urls.modify_order(self._account_id, order['orderId']), json=data, headers=headers)
+        response = requests.post(self._urls.modify_order(self._account_id, order_id), json=data, headers=headers)
 
         return response.json()
 
@@ -537,7 +562,7 @@ class webull:
             response2 = requests.post(self._urls.place_otoco_orders(self._account_id), json=data2, headers=headers)
 
             # print('Resp 2: {}'.format(response2))
-            return response2.text
+            return response2.json()
         else:
             print(result1['checkResultList'][0]['code'])
             print(result1['checkResultList'][0]['msg'])
@@ -567,7 +592,7 @@ class webull:
         response = requests.post(self._urls.modify_otoco_orders(self._account_id), json=data, headers=headers)
 
         # print('Resp: {}'.format(response))
-        return response.text
+        return response.json()
 
     def cancel_order_otoco(self, combo_id=''):
         '''
@@ -717,7 +742,7 @@ class webull:
         response = requests.post(self._urls.place_option_orders(self._account_id), json=data, headers=headers)
         if response.status_code != 200:
             raise Exception('place_option_order failed', response.status_code, response.reason)
-        return response.text
+        return response.json()
 
     def modify_order_option(self, order=None, lmtPrice=None, stpPrice=None, enforce=None, quant=0):
         '''
@@ -761,15 +786,16 @@ class webull:
         for order in open_orders:
             self.cancel_order(order['orderId'])
 
-    def get_tradable(self, stock=''):
+    def get_tradable(self, stock='') :
         '''
         get if stock is tradable
         '''
         headers = self.build_req_headers()
         response = requests.get(self._urls.is_tradable(self.get_ticker(stock)), headers=headers)
+
         return response.json()
 
-    def alerts_list(self):
+    def alerts_list(self) :
         '''
         Get alerts
         '''
@@ -1002,7 +1028,7 @@ class webull:
         headers = self.build_req_headers()
         return requests.get(self._urls.fundamentals(self.get_ticker(stock)), headers=headers).json()
 
-    def get_news(self, stock=None, Id=0, items=20):
+    def get_news(self, stock=None, tId=None, Id=0, items=20):
         '''
         get news and returns a list of articles
         params:
@@ -1010,8 +1036,13 @@ class webull:
             items: number of articles to return
         '''
         headers = self.build_req_headers()
-        params = {'currentNewsId': Id, 'pageSize': items}
-        return requests.get(self._urls.news(self.get_ticker(stock)), params=params, headers=headers).json()
+        if not tId is None:
+            pass
+        elif not stock is None:
+            tId = self.get_ticker(stock)
+        else:
+            raise ValueError('Must provide a stock symbol or a stock id')
+        return requests.get(self._urls.news(tId, Id, items), headers=headers).json()
 
     def get_bars(self, stock=None, tId=None, interval='m1', count=1, extendTrading=0, timeStamp=None):
         '''
@@ -1138,7 +1169,7 @@ class webull:
         return None
 
     def get_dividends(self):
-        ''' Return account's dividend info '''
+        ''' Return account's incoming dividend info '''
         headers = self.build_req_headers()
         data = {}
         response = requests.post(self._urls.dividends(self._account_id), json=data, headers=headers)
@@ -1163,15 +1194,19 @@ class webull:
                     rank = data['data']
         return rank
 
-    def get_watchlists(self) :
+    def get_watchlists(self, as_list_symbols=False) :
         """
         get user watchlists
-        :return:
         """
         headers = self.build_req_headers()
         params = {'version': 0}
         response = requests.get(self._urls.portfolio_lists(), params=params, headers=headers)
-        return response.json()['portfolioList']
+
+        if not as_list_symbols :
+            return response.json()['portfolioList']
+        else:
+            list_ticker = response.json()['portfolioList'][0].get('tickerList')
+            return list(map(lambda x: x.get('symbol'), list_ticker))
 
     def get_account_type(self, username='') :
         try:
@@ -1182,6 +1217,17 @@ class webull:
 
         return account_type
 
+    def is_logged_in(self):
+        '''
+        Check if login session is active
+        '''
+        try:
+            self.get_account_id()
+        except KeyError:
+            return False
+        else:
+            return True
+
 ''' Paper support '''
 class paper_webull(webull):
 
@@ -1219,7 +1265,7 @@ class paper_webull(webull):
         ''' Current positions in paper trading account. '''
         return self.get_account()['positions']
 
-    def place_order(self, stock=None, tId=None, price=0, action='BUY', orderType='LMT', enforce='GTC', quant=0):
+    def place_order(self, stock=None, tId=None, price=0, action='BUY', orderType='LMT', enforce='GTC', quant=0, outsideRegularTradingHour=True):
         ''' Place a paper account order. '''
         if not tId is None:
             pass
@@ -1234,7 +1280,7 @@ class paper_webull(webull):
             'action': action, #  BUY or SELL
             'lmtPrice': float(price),
             'orderType': orderType, # 'LMT','MKT'
-            'outsideRegularTradingHour': True,
+            'outsideRegularTradingHour': outsideRegularTradingHour,
             'quantity': int(quant),
             'serialId': str(uuid.uuid4()),
             'tickerId': tId,
@@ -1248,7 +1294,7 @@ class paper_webull(webull):
         response = requests.post(self._urls.paper_place_order(self._account_id, tId), json=data, headers=headers)
         return response.json()
 
-    def modify_order(self, order, price=0, action='BUY', orderType='LMT', enforce='GTC', quant=0):
+    def modify_order(self, order, price=0, action='BUY', orderType='LMT', enforce='GTC', quant=0, outsideRegularTradingHour=True):
         ''' Modify a paper account order. '''
         headers = self.build_req_headers()
 
@@ -1257,7 +1303,7 @@ class paper_webull(webull):
             'lmtPrice': float(price),
             'orderType':orderType,
             'comboType': 'NORMAL', # 'LMT','MKT'
-            'outsideRegularTradingHour': True,
+            'outsideRegularTradingHour': outsideRegularTradingHour,
             'serialId': str(uuid.uuid4()),
             'tickerId': order['ticker']['tickerId'],
             'timeInForce': enforce # GTC or DAY
